/****************************************************************************
* FILE: poll_interrupt.c
* DESCRIPTION:
*  This example code demonstrates performance differences between polling and 
*  interrupt modes.  On the IBM SP, the default mode is polling.  This can 
*  be overridden by setting the MP_CSS_INTERRUPT environment variable to "yes".
*  Performance should improve when running in interrupt mode.
* LAST REVISED: 01/09/99 Blaise Barney
******************************************************************************/

#include "mpi.h" 
#include <stdio.h>

#define NELEM 8000000
#define REPS 100

int main(argc,argv)
int argc;
char *argv[];
{

int     numtasks, rank, rc, i, j, dest, tag=111, source;
float   arraya[NELEM], arrayb[NELEM], tmp;
double  start, end, result;

MPI_Status status;
MPI_Request reqs[2];

MPI_Init(&argc,&argv);
MPI_Comm_size(MPI_COMM_WORLD,&numtasks);
MPI_Comm_rank(MPI_COMM_WORLD,&rank);

if (numtasks != 2) {
  printf("Please run this test with 2 tasks. Terminating\n");
  MPI_Finalize();
  }

if (rank == 0 ) {
  dest = source = 1;
  printf("\n******************** poll-interrupt ********************\n");
  printf("MP_CSS_INTERRUPT = %s\n",getenv("MP_CSS_INTERRUPT"));
  printf("Doing %d reps with message size= %d bytes\n\n", REPS, sizeof(arraya));

  /* Initialize */
  for (i=0; i<NELEM; i++)
    arraya[i] = arrayb[i] = i * 1.0;
  }

else {
  dest = source = 0;
  }

start = MPI_Wtime();
for (i=0; i<REPS; i++) {
  MPI_Irecv(arrayb, NELEM, MPI_INT, source, tag, MPI_COMM_WORLD, &reqs[0]);
  MPI_Isend(arraya, NELEM, MPI_INT, dest, tag, MPI_COMM_WORLD, &reqs[1]);

  /* use arrayb first */
  MPI_Wait(&reqs[0], &status);
  for (j=0; j<NELEM; j++) {
    tmp = random()*100.0;
    arrayb[i] = arrayb[i] / tmp * 3.98;
    }

  /* now update arraya */
  MPI_Wait(&reqs[1], &status);
  for (j=0; j<NELEM; j++)
    arraya[i] = arraya[i] + arrayb[i];

  if (rank == 0 )
    printf("Did rep #%d\n",i);

  }
end = MPI_Wtime();
printf("Average time per loop= %f seconds.\n",(end-start)/REPS);


MPI_Finalize();
}



